"""Generic page creation functionality.

CONSIDERATIONS:
    Add <!--pyBlog comment/--> tag for insertion of a comment.

TODO:
    OutputPage class.
"""


import os, re

content_header='Content-Type: text/html\n\n'
page_header=content_header #Deprecated

login_redirect="""<html>
    <head>
        <meta http-equiv="refresh" content="5;URL=login.py" />
    </head>
    <body>
        <p>Redirecting to <a href="login.py">login.py</a></p>
    </body>
</html>"""

post_redirect="""<html>
    <head>
        <meta http-equiv="refresh" content="5;URL=post.py" />
    </head>
    <body>
        <p>Redirecting to <a href="post.py">post.py</a></p>
    </body>
</html>"""

class GenericPage:
    """Base page class.  Meant to be subclassed."""
    def __init__(self,UI=1):
        """Fetches interface template and removes UI controls when necessary."""
        template_FILE=open(os.path.join(r'templates','interface.html'),'r')
        self.template=template_FILE.read()
        if not UI: self.removeUI()

    def removeUI(self):
        """Removes UI controls from page."""
        remove_RE=re.compile(r'<!--pyBlog UI-->.*?<!--pyBlog /UI-->',re.DOTALL)
        self.template=remove_RE.sub('',self.template)

    def insert(self,to_insert,XHTML):
        """Replaces <!--pyBlog insert/--> with passed value."""
        return re.sub(r'<!--pyBlog insert/-->',to_insert,XHTML)

    def generateForm(self, form_info):
        """generateForm(self, form_info) -> XHTML
        Takes in a a form_info dictionary and creates an XHTML tag for use
        within a form.

        form_info is a dictionary with one required key (type_tag), one optional
        tag (between_tags), and any number of arbitrary keys.  type_tag is a
        string specifying what type of tag is to be created (input, textarea,
        and select are supported only).  The optional 'between_tags' key
        specifies what (if anything) should be put between the opening and
        closing tags.  The arbitrary keys are for attributes for the tag.

        'input' does not specially use a 'between_tag' key and thus should not
        be used.

        'textarea' optionally takes the 'between_tag' key and blindly inserts
        its value between the opening and closing tags.

        'select' requires the 'between_tag' key and iterates through it
        sequentially (so obviously it must be a sequence), and creates <option>
        tags, all inserted between the opening and closing tags for <select>.

        All tags take any other keys and uses that along with their value to
        create the attributes for the tag in the form of 'key="value"'.

        """
        XHTML=None
        tag_type=form_info['tag_type']
        del form_info['tag_type'] #So as to not accidentally make an attribute in the tag.
        if tag_type=='input':
            XHTML='<input'
            for attribute in form_info.keys():
                XHTML+=' '+attribute+'="'+form_info[attribute]+'"'
            XHTML+=' />'
        elif tag_type=='textarea':
            XHTML='<textarea'
            attributes=form_info.keys()
            if form_info.has_key('between_tags'): attributes.remove('between_tags')
            for attribute in attributes:
                XHTML+=' '+attribute+'="'+form_info[attribute]+'"'
            XHTML+='>'
            if form_info.has_key('between_tags'): XHTML+=form_info['between_tags']
            XHTML+='</textarea>'
        elif tag_type=='select':
            XHTML='<select'
            keys=form_info.keys()
            keys.remove('between_tags')
            for attribute in keys: #Must have 'between_tags' key since select worthless without options.
                XHTML+=' '+attribute+'="'+form_info[attribute]+'"'
            XHTML+='>'
            for option in form_info['between_tags']:
                XHTML+='<option>%s</option>' % option
            XHTML+='</select>'
        return XHTML

    def addFormTags(self, send_to, type):
        """Creates <form> tag."""
        self.template=re.sub(r'<!--pyBlog %s-->' % type,r'<form action="%s" method="post">\n<!--pyBlog %s-->' % (send_to,type), self.template)
        self.template=re.sub(r'<!--pyBlog /%s-->' % type,r'<!--pyBlog /%s-->\n</form>' % type, self.template)

    def generateControls(self,control_info, controls_XHTML):
        """Generates controls for form."""
        generateForm=self.generateForm
        total_controls=''
        for control in control_info:
            total_controls+=generateForm(control)
        return self.insert(total_controls,controls_XHTML)

    

class InputPage(GenericPage):
    """Generates any input page."""
    def __init__(self, send_to, UI=1):
        """Fetches template page, strips out unneeded HTML, and does other general prep work."""
        GenericPage.__init__(self, UI)
        remove_RE=re.compile(r'<!--pyBlog output-->.*<!--pyBlog /output-->',re.DOTALL) #Removes output XHTML to speed up RE operations.
        self.template=remove_RE.sub('',self.template)
        self.addFormTags(send_to,'input')

    def createPage(self,page_info):
        """createPage(self, page_info) -> XHTML
        Creates XHTML for an input page, such as the "login", "add post",
        "archive search" pages, etc.

        Tag hierarchy is as follows (in XML form):
        <input> #Encompassing tag
            <chunk> #Contains each chunk's XHTML
                <title> #Title for form input
                    <insert/> #Insertion point
                </title>
                <form> #XHTML for from input tag
                    <insert/> #Insertion point
                </form>
            </chunk>
            <controls> #XHTML for controls specific to chunks
                <insert/> #Insertion point
            </controls>
        </input>

        page_info is a dictionary with two keys.  One is 'chunks'.  This
        contains a sequence which in turn contains the title/form_info pair in a
        sequence.  Title is a string.  form_info is a dictionary containing all
        attributes for the form input tag to be used by generateForm().  Read
        the documentation for that method for complete details on form and
        requirements for parameters.

        The other key is 'controls'.  This contains a sequence for all the
        information needed for generateForm() to create the controls (such as
        clear and submit buttons) for the page.
        
        """
        input_RE=re.compile(r'<!--pyBlog input-->(?P<XHTML>.*?)<!--pyBlog /input-->',re.DOTALL)
        input_XHTML=input_RE.search(self.template).group('XHTML')
        chunk_RE=re.compile(r'<!--pyBlog chunk-->(?P<XHTML>.*?)<!--pyBlog /chunk-->',re.DOTALL)
        chunk_XHTML=chunk_RE.search(input_XHTML).group('XHTML')
        title_RE=re.compile(r'<!--pyBlog title-->(?P<XHTML>.*?)<!--pyBlog /title-->',re.DOTALL)
        title_XHTML=title_RE.search(chunk_XHTML).group('XHTML')
        form_RE=re.compile(r'<!--pyBlog form-->(?P<XHTML>.*?)<!--pyBlog /form-->',re.DOTALL)
        form_XHTML=form_RE.search(chunk_XHTML).group('XHTML')
        controls_RE=re.compile(r'<!--pyBlog controls-->(?P<XHTML>.*?)<!--pyBlog /controls-->',re.DOTALL)
        controls_XHTML=controls_RE.search(input_XHTML).group('XHTML')

        insert=self.insert
        generateForm=self.generateForm
        chunk_collection=''
        for title,form_info in page_info['chunks']:
            subbed_title=insert(title,title_XHTML)
            subbed_form=insert(generateForm(form_info),form_XHTML)
            subbed_chunk=title_RE.sub(subbed_title,chunk_XHTML)
            subbed_chunk=form_RE.sub(subbed_form,subbed_chunk)
            chunk_collection+=subbed_chunk

        subbed_controls=self.generateControls(page_info['controls'],controls_XHTML)

        final_XHTML=chunk_RE.sub(chunk_collection,self.template)
        final_XHTML=controls_RE.sub(subbed_controls,final_XHTML)
        return final_XHTML




class OutputPage(GenericPage):
    """Generates any output page."""
    def __init__(self, send_to, UI=1):
        """Fetches interface template, strips unneeded HTML, and creates <form> tags."""
        GenericPage.__init__(self,UI)
        remove_RE=re.compile(r'<!--pyBlog input-->.*<!--pyBlog /input-->',re.DOTALL) #Removes input XHTML so as to speed up RE operations.
        self.template=remove_RE.sub('',self.template)
        self.addFormTags(send_to,'output')

    def createPage(self,page_info):
        """createPage(self,page_info) -> XHTML
        Takes in page_info and create XHTML for page.

        page_info is a dictionary with two keys: chunks and controls.

        'chunks' is a sequence of chunks.  The reason it is not a dictionary
        as it is when the post is submitted to the archiver is that
        in this situation order matters.  Even though Python's keys() dictionary
        method always seems to output in the same order for the same keys, it is
        not guaranteed and thus should not be relied upon.

        This also allows you to set the order of the chunks on top of the order
        of the information within the chunks.  This is the same reasoning behind
        why the information is contained within a sequence.

        Each item in the sequence should itself be a sequence with the first
        item being a unique ID and the second a sequence containing the info.
        The unique ID must be a unique identifier to be used by the form to
        identify which chunk was selected by the user.  'info' is a sequence
        containing the info for the chunk.

        The tag hierarchy is as follows (in XML):
        <output> #XHTML for output section.
            <chunk> #XHTML for entire chunk.
                <select> #XHTML for form selection tag.
                    <insert /> #Insertion point for form tag(s)
                    <count /> #Insertion point for number of pieces of info per chunk (used mainly for rowspan for <select>'s <td> tag)
                </select>
                <info> #XHTML for information.
                    <insert /> #Insertion point for info.
                </info>
            </chunk>
        </output>

        An important thing to note is how substitutions in the template is
        handled for <count> after the initial one.  Since <select> affects all
        pieces of information for each chunk but is inserted only once per
        chunk, then the XHTML should also reflect that.  So, after the first
        instance of <info>, all subsequent instances are substituted using the
        XHTML contained in <chunk> and not just in <info>.  This is mainly so
        that tables can be used for the output with <select> having a rowspan
        expanding every row of instance of <select> along the side of every
        instance of <info>.  So taking the template of:
        <!--pyBlog chunk-->
		<tr>
		<!--pyBlog select-->
			<td rowspan="<!--pyBlog count/-->"><!--pyBlog insert/--></td>
		<!--PyBlog /select-->
		<!--pyBlog info-->
			<td><p><!--pyBlog insert/--></p></td>
		<!--pyBlog /info-->
		</tr>
	<!--pyBlog /chunk-->

	can lead to:
	<tr>
            <td rowspan="2"><insert type="radio" value="20010712" /></td>
            <td><p>Title of My Post</p></td>
        </tr>
        <tr>
            <td><p>Date of my post</p></td>
        </tr>
        <tr>
            <td><p>Worked on OutputPage.createPage() documentation today...</p></td>
        </tr>

        'controls' contains a sequence of dictionaries used by
        self.generateForm() to create the tags needed for the form controls.
        Read the documentation for generateForms() for information on how to
        format the dictionary.
        
        """
        output_RE=re.compile(r'<!--pyBlog output-->(?P<XHTML>.*?)<!--pyBlog /output-->',re.DOTALL)
        output_XHTML=output_RE.search(self.template).group('XHTML')
        chunk_RE=re.compile(r'<!--pyBlog chunk-->(?P<XHTML>.*?)<!--pyBlog /chunk-->',re.DOTALL)
        chunk_XHTML=chunk_RE.search(output_XHTML).group('XHTML')
        select_RE=re.compile(r'<!--pyBlog select-->(?P<XHTML>.*?)<!--pyBlog /select-->',re.DOTALL)
        select_XHTML=select_RE.search(chunk_XHTML).group('XHTML')
        car_info_RE=re.compile(r'<!--pyBlog info-->(?P<XHTML>.*?)<!--pyBlog /info-->',re.DOTALL)
        car_info_XHTML=car_info_RE.search(chunk_XHTML).group('XHTML')
        cdr_info_XHTML=select_RE.sub('',chunk_XHTML)
        cdr_info_XHTML=re.sub(r'<!--pyBlog info-->','',cdr_info_XHTML)
        cdr_info_XHTML=re.sub(r'<!--pyBlog /info-->','',cdr_info_XHTML)
        controls_RE=re.compile(r'<!--pyBlog controls-->(?P<XHTML>.*?)<!--pyBlog /controls-->',re.DOTALL)
        controls_XHTML=controls_RE.search(output_XHTML).group('XHTML')

        chunk_collection=''
        insert=self.insert
        generateForm=self.generateForm
        count_RE=re.compile(r'<!--pyBlog count/-->')
        for ID,info in page_info['chunks']:
            subbed_select=insert(generateForm({'tag_type':'input','type':'radio','name':'ID','value':ID}),select_XHTML)
            subbed_select=count_RE.sub(str(len(info)),subbed_select)
            subbed_car_info=insert(info[0],car_info_XHTML)
            subbed_chunk=select_RE.sub(subbed_select,chunk_XHTML)
            chunk_collection+=car_info_RE.sub(subbed_car_info,subbed_chunk)
            for info_piece in info[1:]:
                chunk_collection+=insert(info_piece,cdr_info_XHTML)

        subbed_controls=self.generateControls(page_info['controls'],controls_XHTML)

        final_XHTML=chunk_RE.sub(chunk_collection,self.template)
        final_XHTML=controls_RE.sub(subbed_controls,final_XHTML)
        return final_XHTML
