"""Implements password security; both locally and over the Internet using cookies.

The Security class reads a security.conf file which contains username/password
info (with the password encrypted using the SHA algorithm) and username/
permissions.  Permission support is rough and has no checking features for
validity of values; that functionality might be added in the future.

The InternetSecurity class superclasses Security and adds support for cookies.
Passwords are stored as plain-text in the cookie.  No permission support is
contained in this class since the specifics for that support has yet to be
worked out.
"""

import sha, Cookie, ConfigParser, exceptions, shutil, os, re

class SecurityError(exceptions.Exception):
    """Generic exception for Security class"""
    def __init__(self, args=None): self.args=args

class Security:
    """Handles all security function revolving around security.conf"""
    def __init__(self):
        self.conf_location=r'security.conf'
        self.conf=ConfigParser.ConfigParser()
        self.conf.read(self.conf_location)

    def saveConf(self):
        """Saves security.conf ."""
        location=self.conf_location
        shutil.copyfile(location,location+'~')
        conf_file=open(location,'w')
        try: self.conf.write(conf_file)
        except:
            shutil.copyfile(location+'~',location)
            os.remove(location+'~')
            conf_file.close()
            raise
        else:
            os.remove(location+'~')
            conf_file.close()
            
    def generateDigest(self, password):
        """Generates password digest."""
        prefix_seed='pyBlog'
        suffix_seed='!secure=OCF'
        return sha.new(prefix_seed+password+suffix_seed).digest()

    def checkPasswd(self, username, password):
        """Verifies <password> matches stored password for <username> in security.conf ."""
        if not self.conf.has_option('passwords',username): return 0
        else:
            password_digest=self.generateDigest(password)
            if password_digest==self.conf.get('passwords',username): return 1
            else: return 0

    def checkPermissions(self,username,request=None):
        pass

    def setPasswd(self, username,password):
        """Sets the password for <username>."""
        self.conf.set('passwords',username, self.generateDigest(password))
        self.saveConf()

    def setPermissions(self,username, permission):
        """Sets the permissions for <username>."""
        self.conf.set('permissions',username,permission)
        self.saveConf()
        return 1

    def createUser(self, username, password, permissions=None):
        """Creates an account."""
        unsafe_RE=re.compile(r'\W')
        if self.checkInput(username) and self.checkInput(password):
            self.setPasswd(username,password)
            self.setPermissions(username,None)
            return 1
        else: return 0

    def checkInput(self,stuff):
        """Checkes <stuff> to make sure it does not contain any slashes or any other improper characters that could be used in a buffer overflow exploit."""
        if re.search(r'\W',stuff): return 0
        else: return 1





class InternetSecurity(Security):
    """Handles all cookie-related security."""
    def __init__(self):
        Security.__init__(self)
        self.getCookie()

    def getCookie(self):
        """Fetches the user's cookie."""
        C=Cookie.SimpleCookie()
        if os.environ.has_key('HTTP_COOKIE'): C.load(os.environ['HTTP_COOKIE'])
        self.C=C
        
    def expireCookie(self):
        """Expires the user's cookie."""
        self.expireCookiePasswd()

    def expireCookiePasswd(self):
        """Expires user's cookie.
        This is done by setting the max-age to 0, thus causing the cookie to
        disappear after this browsing session."""
        C=Cookie.SimpleCookie()
        C['username']=None
        C['username']['max-age']=0
        C['password']=None
        C['password']['max-age']=0
        self.C=C
        return C.output()

    def checkCookiePasswd(self):
        """Verifies the password cookie matches the stored password for the user specified in the user cookie."""
        C=self.C
        try:
            user_password=C['password'].value
            user_name=C['username'].value
        except KeyError: return 0
        if self.checkInput(user_name) and self.checkInput(user_password):
            return self.checkPasswd(user_name,user_password)
        else: return 0
        
    def setCookiePasswd(self,username,password,max_age=None,secure=None):
        """Sets the user and password cookie.
        Both are set to max-age of 0 for security concerns."""
        C=self.C
        if self.checkInput(username) and self.checkInput(password):
            C['username']=username
            C['password']=password
            C['username']['max-age']=0
            C['password']['max-age']=0
            return C.output()
        else: return self.expireCookiePasswd()
